@@ -1,4 +1,37 @@
+// BSD 3-Clause License
+//
+// Copyright © 2021 Keegan Saunders
+// Copyright © 2021 VTIL Project
+// All rights reserved.
+//
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are met:
+//
+// 1. Redistributions of source code must retain the above copyright notice, this
+//    list of conditions and the following disclaimer.
+//
+// 2. Redistributions in binary form must reproduce the above copyright notice,
+//    this list of conditions and the following disclaimer in the documentation
+//    and/or other materials provided with the distribution.
+//
+// 3. Neither the name of the copyright holder nor the names of its
+//    contributors may be used to endorse or promote products derived from
+//    this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+//
+
 use crate::{arch_info, Error, Result};
+use indexmap::map::IndexMap;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Deserializer, Serialize, Serializer};
 use std::{convert::TryInto, fmt};
@@ -25,10 +58,17 @@ pub struct Header {
 
 /// VTIL instruction pointer
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 #[repr(transparent)]
 pub struct Vip(pub u64);
 
+impl Vip {
+    /// Invalid instruction pointer, unassociated with [`BasicBlock`]
+    pub fn invalid() -> Vip {
+        Vip(!0)
+    }
+}
+
 bitflags! {
     /// Flags describing register properties
     #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
@@ -63,10 +103,10 @@ bitflags! {
 /// Describes a VTIL register in an operand
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, Copy)]
-pub struct Reg {
+pub struct RegisterDesc {
     /// Flags describing the register
     pub flags: RegisterFlags,
-    /// Identifier for this register, use [`Reg::local_id`]
+    /// Identifier for this register, use [`RegisterDesc::local_id`]
     pub combined_id: u64,
     /// The bit count of this register (e.g.: 32)
     pub bit_count: i32,
@@ -74,7 +114,7 @@ pub struct Reg {
     pub bit_offset: i32,
 }
 
-impl Reg {
+impl RegisterDesc {
     /// Local identifier that is intentionally unique to this register
     pub fn local_id(&self) -> u64 {
         self.combined_id & !(0xff << 56)
@@ -91,7 +131,7 @@ impl Reg {
     }
 }
 
-impl fmt::Display for Reg {
+impl fmt::Display for RegisterDesc {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         let mut prefix = String::new();
 
@@ -166,19 +206,19 @@ impl fmt::Display for Reg {
 
 /// Routine calling convention information and associated metadata
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct RoutineConvention {
     /// List of registers that may change as a result of the routine execution but
     /// will be considered trashed
-    pub volatile_registers: Vec<Reg>,
+    pub volatile_registers: Vec<RegisterDesc>,
     /// List of regsiters that this routine wlil read from as a way of taking arguments
     /// * Additional arguments will be passed at `[$sp + shadow_space + n * 8]`
-    pub param_registers: Vec<Reg>,
+    pub param_registers: Vec<RegisterDesc>,
     /// List of registers that are used to store the return value of the routine and
     /// thus will change during routine execution but must be considered "used" by return
-    pub retval_registers: Vec<Reg>,
+    pub retval_registers: Vec<RegisterDesc>,
     /// Register that is generally used to store the stack frame if relevant
-    pub frame_register: Reg,
+    pub frame_register: RegisterDesc,
     /// Size of the shadow space
     pub shadow_space: u64,
     /// Purges any writes to stack that will be end up below the final stack pointer
@@ -243,26 +283,26 @@ impl fmt::Debug for Immediate {
 /// Describes a VTIL immediate value in an operand
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, Copy)]
-pub struct Imm {
+pub struct ImmediateDesc {
     pub(crate) value: Immediate,
     /// The bit count of this register (e.g.: 32)
     pub bit_count: u32,
 }
 
-impl Imm {
+impl ImmediateDesc {
     /// Immediate from a `u64`
-    pub fn new<T: Into<u64>>(value: T, bit_count: u32) -> Imm {
+    pub fn new<T: Into<u64>>(value: T, bit_count: u32) -> ImmediateDesc {
         assert!(bit_count % 8 == 0);
-        Imm {
+        ImmediateDesc {
             value: Immediate { u64: value.into() },
             bit_count,
         }
     }
 
     /// Immediate from an `i64`
-    pub fn new_signed<T: Into<i64>>(value: T, bit_count: u32) -> Imm {
+    pub fn new_signed<T: Into<i64>>(value: T, bit_count: u32) -> ImmediateDesc {
         assert!(bit_count % 8 == 0);
-        Imm {
+        ImmediateDesc {
             value: Immediate { i64: value.into() },
             bit_count,
         }
@@ -294,34 +334,46 @@ impl Imm {
 #[derive(Debug, Clone, Copy)]
 pub enum Operand {
     /// Immediate operand containing a sized immediate value
-    Imm(Imm),
+    ImmediateDesc(ImmediateDesc),
     /// Register operand containing a register description
-    Reg(Reg),
+    RegisterDesc(RegisterDesc),
 }
 
-impl<'a, 'b> TryInto<&'b Imm> for &'a Operand
+impl From<RegisterDesc> for Operand {
+    fn from(register_desc: RegisterDesc) -> Self {
+        Operand::RegisterDesc(register_desc)
+    }
+}
+
+impl From<ImmediateDesc> for Operand {
+    fn from(immediate_desc: ImmediateDesc) -> Self {
+        Operand::ImmediateDesc(immediate_desc)
+    }
+}
+
+impl<'a, 'b> TryInto<&'b ImmediateDesc> for &'a Operand
 where
     'a: 'b,
 {
     type Error = Error;
 
-    fn try_into(self) -> Result<&'a Imm> {
+    fn try_into(self) -> Result<&'a ImmediateDesc> {
         match self {
-            Operand::Imm(ref i) => Ok(i),
+            Operand::ImmediateDesc(ref i) => Ok(i),
             _ => Err(Error::OperandTypeMismatch),
         }
     }
 }
 
-impl<'a, 'b> TryInto<&'b Reg> for &'a Operand
+impl<'a, 'b> TryInto<&'b RegisterDesc> for &'a Operand
 where
     'a: 'b,
 {
     type Error = Error;
 
-    fn try_into(self) -> Result<&'a Reg> {
+    fn try_into(self) -> Result<&'a RegisterDesc> {
         match self {
-            Operand::Reg(r) => Ok(r),
+            Operand::RegisterDesc(r) => Ok(r),
             _ => Err(Error::OperandTypeMismatch),
         }
     }
@@ -655,13 +707,27 @@ pub struct BasicBlock {
     pub next_vip: Vec<Vip>,
 }
 
+impl BasicBlock {
+    /// Allocate a temporary register for this basic block
+    pub fn tmp(&mut self, bit_count: i32) -> RegisterDesc {
+        let reg = RegisterDesc {
+            flags: RegisterFlags::LOCAL,
+            combined_id: self.last_temporary_index as u64,
+            bit_count,
+            bit_offset: 0,
+        };
+        self.last_temporary_index += 1;
+        reg
+    }
+}
+
 /// Alias for [`RoutineConvention`] for consistent naming
 pub type SubroutineConvention = RoutineConvention;
 
-/// VTIL container
+/// VTIL routine container
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 #[derive(Debug)]
-pub struct VTIL {
+pub struct Routine {
     /// Header containing metadata about the VTIL container
     pub header: Header,
     /// The entry virtual instruction pointer for this VTIL routine
@@ -673,5 +739,5 @@ pub struct VTIL {
     /// All special subroutine calling conventions in the top-level VTIL routine
     pub spec_subroutine_conventions: Vec<SubroutineConvention>,
     /// Reachable [`BasicBlock`]s generated during a code-discovery analysis pass
-    pub explored_blocks: Vec<BasicBlock>,
+    pub explored_blocks: IndexMap<Vip, BasicBlock>,
 }
